//fixed bugs today. 04/09
//still need to change CA rules.

import java.awt.*;

class NdRoad {
  // Read Only variables for display:

  // length of the road
  public int LENGTH = 400;
  // Maximum speed
  public int MAXSPEED;
  // the road as an array of speeds
  public int[] speed;
  public int[] local_speed;
  // local density of vehicles on the road [veh/site]
  public double dens;
  // local mean velocity [sites/timestep]
  public double v;

  // global frequency of gaps
  public double[] gapfreq;
  // global frequency of velocities
  public double[] vfreq;


  // number of vehicles on the road
  private int cars;
  // cell length
  private double deltaX;
  private int[] ve;

  private int[][][] rules;

  // my private colors
  private final Color streetcolor = Color.gray;
  private final Color background = Color.white;
  private final Color foreground = Color.black;
  private final Color linecolor = Color.blue;
  private final Color vcolor[];

  private static final int LEFT   =  0;
  private static final int RIGHT  =  2;
  private static final int CENTER = 1;
  private static final int VMAX   = 1;


  public NdRoad (double density, double deltaX) {
    this.deltaX   = deltaX;
    this.MAXSPEED = (int)(135.0/(deltaX*3.6)) + 1; // 135 km/h, dt=1s

    // the road as an array of speeds
    speed  = new int[LENGTH];
    cars   = 0;
    rules  = new int[MAXSPEED+1][MAXSPEED+1][MAXSPEED+1];

    for(int i=0;i<=MAXSPEED;i++)
    {
      switch(i){
      case 0:
        //rules[0][i][0] = MAXSPEED;
        rules[MAXSPEED][i][MAXSPEED] = i + VMAX;
        for(int j=0;j<MAXSPEED;j++){
          //set speed = 1, don't matter what speed the car
          //ahead is, as long as > 0;
          rules[j][i][0] = 0;
          for(int k=1;k<MAXSPEED;k++)
          {
            //in the case of speed = 0, take the average speed of
            //the neighboring cells
            /*if(k >= j && j > 0)
              rules[j][i][k] = (k + j)/2;
            else
              rules[j][i][k] = k;
            */
           rules[j][i][k] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
           rules[MAXSPEED][i][k] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
          }
          rules[j][i][MAXSPEED] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
        }
        break;
      case 1:
        rules[MAXSPEED][i][MAXSPEED] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
        for(int j=0;j<MAXSPEED;j++){
          //speed up
          //set speed = 1, don't matter what speed the car
          //ahead is, as long as > 0;
          rules[j][i][0] = 0;
          for(int k=1;k<MAXSPEED;k++)
          {
            /*if(k >= j && j> 0)
              rules[j][i][k] = ((k+j)/ 2 > i) ? i: (k+j)/2;
           else
              rules[j][i][k] = k;
            */
            if (i == k){
              rules[j][i][k] = i;
              rules[MAXSPEED][i][k] = i;
            }
            else{
              rules[j][i][k] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
              rules[MAXSPEED][i][k] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
            }
          }
          rules[j][i][MAXSPEED] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
        }
        break;
      default:
        overLimit(rules, i);
      break;
    }
    }
    // clear road
    for (int i=0;i<LENGTH;i++) {
      speed[i] = 0;
    }
    /*for(int i =0;i<=MAXSPEED;i++)
      for(int j=0;j<=MAXSPEED;j++)
        for(int k=0;k<=MAXSPEED;k++)
        {
          if(rules[i][j][k] == MAXSPEED)
          {
            System.out.println("i: " + i + " j: " + j + " k: " + k + " " + rules[i][j][k]);
            }
        }
    */
    vcolor = new Color[this.MAXSPEED + 1];
    // colors for different velocities
    for (int i= 0; i<vcolor.length; i++) {
      switch(i) {
      case 0: vcolor[i] = Color.red;      break;
      case 1: vcolor[i] = Color.blue;     break;
      case 2: vcolor[i] = Color.orange;   break;
      case 3: vcolor[i] = Color.cyan;     break;
      case 4: vcolor[i] = Color.green;    break;
      case 5: vcolor[i] = Color.yellow;   break;
      default:
        vcolor[i] = new Color(200,200,200);
    }
    }
  }
  public void calculate(int[][][] rules, int i){
    rules[MAXSPEED][i][MAXSPEED] = (i+1 >= MAXSPEED) ? MAXSPEED - 1 : i + 1;
    for(int k=0;k<=MAXSPEED;k++)
      rules[k][i][0] = i - 1;
    for(int j=0;j<=MAXSPEED;j++){
        for(int k=1;k<MAXSPEED;k++)
        {
          if(i > k){
            rules[j][i][k] = i - 2;
            rules[MAXSPEED][i][k] = i - 2;
          }
          else{
            rules[j][i][k] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
            rules[MAXSPEED][i][k] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
          }
        }
        rules[j][i][MAXSPEED] = (i + VMAX > MAXSPEED) ? i : i + VMAX;
      }
  }

  public void overLimit(int[][][] rules, int i){
    if(i > 1 && i < MAXSPEED){
      for(int x = 2; x < MAXSPEED; x++)
        calculate(rules, x);
      return;
    }

    for(int j=0;j<=MAXSPEED;j++){
      for(int k=0;k<=MAXSPEED;k++)
        rules[j][i][k] = MAXSPEED;
    }

  }
  public void update(double enterRate, double leaveRate)
  {
    int x=0;
    try{
      for(; x<LENGTH; x++)
        speed[x] = local_speed[x];
    }
    catch (ArrayIndexOutOfBoundsException ex1) {
      //System.out.println("k: " + x + " speed " + speed[x]);
      speed[x] = MAXSPEED;
    }
    catch (NullPointerException ex2){
     speed[x] = MAXSPEED;
    }
    int i = LENGTH-2;
    //looking for the first car on the road.

  try{
      while((i > 0) && ((speed[i] == -1 || speed[i] >= MAXSPEED)))
        i--;
      while(i> 1)
      {
        //need to take care of corner case here, do we want to wrap around?
        //need to clean this up.

        if(speed[i] > 0 && speed[i] < MAXSPEED)
          speed[i] = makeRules(speed[i], speed[i-1], speed[i+1]);

        //i-=(gap+1);
        i--;
      }


      } catch (ArrayIndexOutOfBoundsException ex1) {
        //System.out.println("i: " + i + " speed " + speed[i]);
        speed[i] = MAXSPEED;
      }


      // move the vehicles.
      // where is the 1st vehicle
      local_speed = new int[LENGTH];
      i = -1;
      // go on until reaching the end of the lane
      while (++i < LENGTH-2) {
        local_speed[i+1] = speed[i];

      }

    local_speed[0] = (Math.random() < 0.2) ? 1 : MAXSPEED;

    i = 0;
    while(++ i< LENGTH - 2)
    {
      if(speed[i] == -1 || speed[i] >= MAXSPEED)
      {
        local_speed[i] = (Math.random() <= enterRate) ? 1 : -1;
      }
      else
        local_speed[i] = (Math.random() <= leaveRate) ? -1: local_speed[i];
    }

    //for(int y=0;y < LENGTH;y++)
     // System.out.print(speed[y]);
    //System.out.println("\n");


  }

  public int makeRules(int centerCell, int leftNeighbor, int rightNeighbor)
  {
    //determine rules for each cell.
    if(centerCell < 0)
      centerCell = MAXSPEED;
    if(leftNeighbor < 0)
      leftNeighbor = MAXSPEED;
    if(rightNeighbor < 0)
      rightNeighbor = MAXSPEED;
    return rules[leftNeighbor][centerCell][rightNeighbor];

  }

  public void paint(Graphics g, int row,
                                 int dotdist, int dotsize, int xshift) {
       int i, j, num, local_speed;

       for (i = 0; i < 400; i++) {
         num = 0; local_speed = 0;
         //for (j = 0; j < combine; j++) {
           if (speed[i] >= 0) {
             local_speed += speed[i];
             num++;
          // }
           }
         if (num > 0) {
           g.setColor(vcolor[local_speed/num]);
           g.fillRect((xshift+i)*dotdist,row,dotsize,dotsize);
         }
       }
       g.setColor(background);
   }

   public void setPreviousRules(int[] speed)
   {
    local_speed = speed;
   }
}